"""
Classifies: CHEBI:134019 hydroperoxy polyunsaturated fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_hydroperoxy_polyunsaturated_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is a hydroperoxy polyunsaturated fatty acid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroperoxy polyunsaturated fatty acid anion, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylate anion
    carboxylate_pattern = Chem.MolFromSmarts('[C](=[O])[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion group found"

    # Check for hydroperoxy group (-OOH or -OO-)
    hydroperoxy_pattern = Chem.MolFromSmarts('[C,c]OO')
    if not mol.HasSubstructMatch(hydroperoxy_pattern):
        return False, "No hydroperoxy group found"

    # Count conjugated double bonds in carbon chain
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    matches = mol.GetSubstructMatches(double_bond_pattern)
    if len(matches) < 2:
        return False, "Not polyunsaturated (less than 2 double bonds)"

    # Check if it's a fatty acid by verifying the presence of a long aliphatic chain
    aliphatic_chain = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetIsAromatic() == False:
            chain = Chem.FindAtomEnvironmentOfRadiusN(mol, 6, atom.GetIdx())
            if chain:
                atoms_in_chain = set()
                for bid in chain:
                    bond = mol.GetBondWithIdx(bid)
                    atoms_in_chain.add(bond.GetBeginAtomIdx())
                    atoms_in_chain.add(bond.GetEndAtomIdx())
                if len(atoms_in_chain) >= 8:
                    aliphatic_chain = True
                    break

    if not aliphatic_chain:
        return False, "No long aliphatic chain characteristic of fatty acids"

    # Additional check to filter out non-fatty acid structures
    ring_info = mol.GetRingInfo()
    if ring_info.NumRings() > 1:  # Allow max 1 ring as some fatty acids can be cyclic
        return False, "Structure contains too many rings for a fatty acid"

    return True, f"Contains carboxylate anion, hydroperoxy group, and aliphatic chain with multiple double bonds"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134019',
                          'name': 'hydroperoxy polyunsaturated fatty acid '
                                  'anion',
                          'definition': 'Any polyunsaturated fatty acid anion '
                                        'carrying one or more hydroperoxy '
                                        'substituents.',
                          'parents': ['CHEBI:64012', 'CHEBI:76567']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.13793103448275862 is too low.\n'
               'True positives: '
               "[('CCCCC\\\\C=C/C[C@H](OO)\\\\C=C\\\\CCCCCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), ('CCCCC\\\\C=C/CC(OO)\\\\C=C\\\\C=C/CCCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C(=C\\\\[C@H](C/C=C\\\\C/C=C\\\\CCCCC)OO)/C=C\\\\CCCC(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C=C/[C@H](C\\\\C=C/CCCC([O-])=O)OO', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('C(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\C(CCCC(=O)[O-])OO)CC', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('CC\\\\C=C/C\\\\C=C/CC(OO)\\\\C=C\\\\C=C/C\\\\C=C/CCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('[O-]C(CC/C=C\\\\CC/C=C/C=C\\\\C=C\\\\[C@H](C/C=C\\\\C/C=C\\\\CC)OO)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('C(=C\\\\[C@@H](/C=C\\\\CCCCC)OO)\\\\CCCCCCCC(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds')]\n"
               'False positives: '
               "[('CC(C)CCCCCC(=O)OC(CC([O-])=O)C[N+](C)(C)C', 'Contains "
               "carboxylate anion, hydroperoxy group, and 2 double bonds'), "
               "('[C@H]1(O[C@@H]([C@H](O)[C@@H]([C@H]1O)O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)CO)O[C@@H]3[C@H]([C@H](O[C@@H]4[C@H]([C@H](O[C@@H]5[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]5O)O)CO)O[C@@H]([C@@H]4O)CO[C@H]6[C@@H]([C@H]([C@H](O[C@@H]7O[C@@H]([C@H](O)[C@@H]([C@H]7O)O)CO)[C@H](O6)CO)O[C@@H]8O[C@H]([C@@H](O)[C@H]([C@@H]8O)O)C)NC(C)=O)O)O[C@H](CO)[C@H]3O[C@@H]9O[C@H]([C@@H](O)[C@H]([C@@H]9O)O)C)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('[NH3+][C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@@H]1O[C@@H](CC([O-])=O)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O[C@@]5(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O5)[C@H](O)[C@H](O)CO)C([O-])=O)[C@H]4O)[C@H]3NC(C)=O)[C@H](O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@@H](CO)O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@H](O)CO)C([O-])=O)C([O-])=O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 9 double "
               "bonds'), "
               "('C1[C@H]2[C@@H]([C@H]([C@@H]1OO2)/C=C/[C@H](CCCCC)O)CCCCCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('O(C[C@@H]([C@@H](CCCCCCCCCCCCCCC)O)NC(=O)*)[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)CO)O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O)O[C@H]3[C@@H]([C@H]([C@@H]([C@H](O3)CO)O[C@H]4[C@@H]([C@H]([C@H]([C@H](O4)CO)O)O[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)CO)O[C@@H]6O[C@@H]([C@@H]([C@@H]([C@H]6O)O[C@]7(O[C@]([C@@H]([C@H](C7)O)NC(CO)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)O)NC(=O)C)O)O[C@@H]8O[C@H]([C@H]([C@H]([C@@H]8O)O)O)C)NC(C)=O)O)O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('C[C@]1(CC([O-])=O)[C@H](CCC([O-])=O)\\\\C2=C\\\\c3[nH]c(Cc4[nH]c(c(CC([O-])=O)c4CCC([O-])=O)[C@](C)(O)[C@@]45N\\\\C(=C/C1=[NH+]2)[C@@H](CCC([O-])=O)[C@]4(C)CC(=O)O5)c(CCC([O-])=O)c3CC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 11 double "
               "bonds'), "
               "('[H][C@]1(O[C@@](C[C@H](O)[C@H]1NC(C)=O)(OC[C@H]1O[C@@H](O[C@H]2[C@H](O)[C@@H](O)[C@H](OC[C@H](NC([*])=O)[C@H](O)\\\\C=C\\\\CCCCCCCCCCCCC)O[C@@H]2CO)[C@H](O)[C@@H](O)[C@H]1O)C([O-])=O)[C@H](O)[C@H](O)CO', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C(O[C@]1(O[C@@]([C@@H]([C@H](C1)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])[C@H]2O[C@H]([C@@H]([C@H]([C@H]2O)O)O)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)CO)O[C@H]6[C@@H]([C@H]([C@H]([C@H](O6)CO[C@]7(O[C@@]([C@@H]([C@H](C7)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])O)O)O)O)NC(C)=O)O[C@H]8[C@@H]([C@H]([C@@H](O[C@@H]8CO)O[C@@H]9[C@H]([C@@H](O[C@@H]([C@@H]9O)CO)O[C@H]%10[C@@H]([C@H]([C@@H](O[C@@H]%10CO)OC[C@@H]([C@@H](*)O)NC(=O)*)O)O)O)NC(C)=O)O)O)NC(C)=O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O[C@@H]5O[C@@H]([C@@H]([C@@H]([C@H]5O)O[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O[C@]7(O[C@]([C@@H]([C@H](C7)O)NC(C)=O)([C@@H]([C@@H](COC(=O)C)O)O)[H])C([O-])=O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 9 double "
               "bonds'), ('O([C@@H](C[N+](C)(C)C)CC([O-])=O)C(=O)CCCCCCCCCC', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C(O[C@]1(O[C@@]([C@@H]([C@H](C1)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])[C@H]2O[C@H]([C@@H]([C@H]([C@H]2O)O)O)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)CO)O[C@H]6[C@@H]([C@H]([C@H]([C@H](O6)CO[C@]7(O[C@@]([C@@H]([C@H](C7)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])O)O)O)O)NC(C)=O)O[C@H]8[C@@H]([C@H]([C@@H](O[C@@H]8CO)O[C@@H]9[C@H]([C@@H](O[C@@H]([C@@H]9O)CO)O[C@H]%10[C@@H]([C@H]([C@@H](O[C@@H]%10CO)OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)O)O)O)NC(C)=O)O)O)NC(C)=O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)O*)OP(OC[C@@H]([C@@H](O*)O)NC(*)=O)(=O)[O-])O[C@@H]2[C@H]([NH3+])[C@@H](O)[C@@H]([C@H](O2)CO)O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO[C@@H]4[C@H]([C@H]([C@@H]([C@H](O4)CO*)O)O)O[C@@H]5[C@H]([C@H]([C@@H]([C@H](O5)COP([O-])(=O)OCCNC([C@@H](N*)CC(=O)[O-])=O)O)O)O*)O*)O*)O*', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('P(=O)(OC[C@H](OC(CCCCCCCCCCCCCCC)=O)COC(CCCCCCCCCCCCCCC)=O)(OC[C@@H](C([O-])=O)[NH3+])[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C1[C@H]2[C@@H]([C@H](O[C@@H]1C2)/C=C/[C@H](CCCC(C)O)O)C/C=C\\\\CCCC(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H](*)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O[C@H]5[C@@H]([C@@H](O[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@H](O[C@]7(O[C@]([C@@H]([C@H](C7)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C(=O)[O-])CO)O)[H])C([O-])=O)[C@H]([C@H](O5)CO)O)O)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O[C@]8(O[C@]([C@@H]([C@H](C8)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 10 double "
               "bonds'), "
               "('C[C@H]1O[C@H](O[C@H]2[C@H](O)[C@H](NC(C)=O)[C@H](O[C@H]3[C@H](O)[C@@H](NC(C)=O)[C@H](O[C@@H]3CO)OP([O-])(=O)OP([O-])(=O)OC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(\\\\C)CC\\\\C=C(/C)CC\\\\C=C(/C)CCC=C(C)C)O[C@@H]2C([O-])=O)[C@H](O)[C@@H](O)[C@H]1NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 17 double "
               "bonds'), "
               "('[C@@H]1([C@@H]([C@H]([C@H]([C@H](O1)CO)O)O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(CO)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO)O[C@H]5[C@@H]([C@H]([C@@H](O[C@@H]5CO)OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCCCC)O)NC(=O)*)O)O)O)NC(C)=O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[C@H]1(O[C@@H]([C@@H]([C@@H]([C@H]1O)O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO)O[C@H]5[C@@H]([C@H]([C@@H](O[C@@H]5CO)O[C@@H]6[C@H]([C@@H](O[C@@H]([C@@H]6O)CO)O[C@H]7[C@@H]([C@H]([C@@H](O[C@@H]7CO)OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)O)O)O)NC(C)=O)O)O)NC(=O)C)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('O([C@@H]1[C@H]([C@@H](O[C@@H]([C@@H]1O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O)O)NC(=O)C)CO)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO)O[C@H]5[C@@H]([C@H]([C@@H](O[C@@H]5CO)OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCCCC)O)NC(=O)*)O)O)O)NC(=O)C)O)O)[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('[C@@]1(O[C@]([C@@H]([C@H](C1)O)NC(C)=O)([C@@H]([C@H](O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(C)=O)([C@@H]([C@@H](CO)O[C@]3(O[C@]([C@@H]([C@H](C3)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C(=O)[O-])O)[H])C(=O)[O-])CO)O)[H])(C([O-])=O)O[C@H]4[C@H]([C@H](O[C@H]([C@@H]4O)O[C@@H]5[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]5O)O)CO)CO)O[C@H]6[C@@H]([C@H]([C@@H](O)[C@H](O6)CO)O[C@H]7[C@@H]([C@@H](O)[C@H]([C@H](O7)CO)O)O)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 9 double "
               "bonds'), "
               "('C([C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(*)=O)O[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)CO)O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@H]([C@H](O4)CO)O)O)O)NC(C)=O)O[C@@]5(C[C@@H]([C@H]([C@@](O5)([C@@H]([C@@H](CO)O)O)[H])NC(CO)=O)O)C([O-])=O)O)O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O)[C@H](O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@H](O)CO)C([O-])=O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('Nc1c([nH+]cn1[C@@H]1O[C@H](COP([O-])([O-])=O)[C@@H](O)[C@H]1O)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('[H][C@]1(O[C@@](C[C@H](O)[C@H]1NC(C)=O)(O[C@H]1[C@@H](O)[C@@H](CO)O[C@@H](OC[C@H](NC([*])=O)[C@H](O)[*])[C@@H]1O)C([O-])=O)[C@H](O)[C@H](O)CO', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@H]1([C@@H]([C@H]([C@H]([C@H](O1)CO[C@H]2[C@@H]([C@H]([C@@H]([C@H](O2)CO)O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@@H]([C@H](O4)CO)O[C@H]5[C@@H]([C@H]([C@H]([C@H](O5)CO)O)O[C@@H]6[C@@H]([C@H]([C@H]([C@H](O6)CO)O)O)NC(C)=O)O[C@H]7[C@H]([C@@H]([C@@H]([C@@H](O7)C)O)O)O)O[C@H]8[C@H]([C@@H]([C@@H]([C@@H](O8)C)O)O)O)NC(C)=O)O)O)NC(C)=O)O)O[C@H]9[C@@H]([C@H]([C@@H]([C@H](O9)CO)O[C@H]%10[C@@H]([C@H]([C@H]([C@H](O%10)CO)O)O[C@]%11(O[C@]([C@@H]([C@H](C%11)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)O)NC(C)=O)O)O[C@H]%12[C@@H]([C@H]([C@@H](O[C@@H]%12CO)O[C@@H]%13[C@H]([C@@H](O[C@@H]([C@@H]%13O)CO)O[C@H]%14[C@@H]([C@H]([C@@H](O[C@@H]%14CO)OC[C@@H]([C@@H](*)O)NC(=O)*)O)O)O)NC(C)=O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('O([C@H]1[C@H]([C@@H]([C@@H]([C@@H](O1)C)O)O)O)[C@@H]2[C@H]([C@@H](O[C@@H]([C@H]2O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@@H]([C@H](O4)CO)O[C@H]5[C@H]([C@@H]([C@@H]([C@@H](O5)C)O)O)O)O[C@H]6[C@@H]([C@H]([C@H]([C@H](O6)CO)O)O[C@]7(O[C@@]([C@@H]([C@H](C7)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])O)NC(C)=O)O)CO)O[C@@H]8[C@H]([C@@H](O[C@@H]([C@@H]8O)CO)O[C@H]9[C@@H]([C@H]([C@@H](O[C@@H]9CO)OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCCCC)O)NC(=O)*)O)O)O)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('CC(C)CCC[C@@H](C)CCC[C@@H](C)CCC[C@@H](C)CCOC[C@@H](COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O)OCC[C@H](C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('[Na+].O(C1C2C(C(C=CC2=CC(C1)C)C)CCC(O)CC(O)CC([O-])=O)C(=O)C(CC)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('[C@H]12[C@H](C[C@H]([C@@H]1/C=C\\\\C/C=C\\\\C/C=C\\\\CCCC([O-])=O)/C=C/C)O2', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('O[C@H]1[C@H]([C@H](O[C@H]([C@@H]1O)O[C@@H]2[C@H](O[C@@H](OC[C@@H]([C@@H](CCCCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]2O)O)CO)CO)O[C@H]3[C@@H]([C@H]([C@@H](O)[C@H](O3)CO)O[C@H]4[C@@H]([C@@H](O[C@]5(O[C@]([C@@H]([C@H](C5)O)NC(C)=O)([C@@H]([C@H](O[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C(=O)[O-])CO)O)[H])C([O-])=O)[C@H]([C@H](O4)CO)O)O)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('C[C@H](NC(=O)[C@@H](C)O[C@H]1[C@H](O)[C@@H](CO)OC(OP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2O)n2ccc(=O)[nH]c2=O)[C@@H]1NC(C)=O)C(=O)N[C@H](CCC([O-])=O)C(=O)NCCCC[C@@H]([NH3+])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 10 double "
               "bonds'), "
               "('O([C@@H]1[C@H]([C@H](O[C@@H]2[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]2O)O)CO)O[C@@H]([C@@H]1O)CO)O)[C@]3(O[C@]([C@@H]([C@H](C3)O)NC(C)=O)([C@@H]([C@H](O[C@]4(O[C@]([C@@H]([C@H](C4)O)NC(C)=O)([C@@H]([C@@H](COC(C)=O)O)O)[H])C(=O)[O-])CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('[C@H]1(O[C@@H]([C@@H]([C@@H]([C@H]1O)O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(CO)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO)O[C@H]5[C@@H]([C@H]([C@@H](O[C@@H]5CO)O[C@@H]6[C@H]([C@@H](O[C@@H]([C@@H]6O)CO)O[C@H]7[C@@H]([C@H]([C@@H](O[C@@H]7CO)OC[C@@H]([C@@H](*)O)NC(=O)*)O)O)O)NC(C)=O)O[C@@H]8O[C@H]([C@H]([C@H]([C@@H]8O)O)O)C)O)NC(=O)C)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)c1c(OC)cccc1OC)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@H]1(O[C@@H]([C@H](O)[C@@H]([C@H]1O)O[C@@H]2O[C@H](CO)[C@H]([C@@H]([C@H]2NC(=O)C)O)O[C@@H]3O[C@@H]([C@H](O)[C@@H]([C@H]3O)O)CO[C@]4(O[C@]([C@@H]([C@H](C4)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)CO)O[C@H]5[C@@H]([C@@H](NC(=O)C)[C@H](O[C@@H]6[C@H]([C@H](O[C@H]7[C@@H]([C@H]([C@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(*)=O)O[C@@H]7CO)O)O)O[C@@H]([C@@H]6O)CO)O)O[C@@H]5CO)O[C@@H]8O[C@H]([C@H]([C@H]([C@@H]8O)O)O)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('[Na+].COc1c(C)c2COC(=O)c2c(O)c1C\\\\C=C(/C)CCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('O([C@@H](C[N+](C)(C)C)CC([O-])=O)C(=O)CCCCCCC(O)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('O([C@@H]1[C@H]([C@@H](O[C@@H]([C@@H]1O)CO)O[C@H]2[C@@H]([C@H]([C@@H](O[C@@H]2CO)O[C@@H]3[C@H]([C@@H](O[C@@H]([C@@H]3O)CO)O[C@H]4[C@@H]([C@H]([C@@H](O[C@@H]4CO)OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)O)O)O)NC(C)=O)O[C@@H]5O[C@H]([C@H]([C@H]([C@@H]5O)O)O)C)O)[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C=1C=C(C=CC1C[C@](O)(C([O-])=O)[C@H](OC(/C=C/C=2C=CC(=C(C2)OC)O)=O)C([O-])=O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C1(C2=C(C(=CC(=C2)C([O-])=O)O)O)=C(C(=CC(=C1)C(=O)[O-])OC)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C[C@@H]1O[C@@H](OCCCCCCC(=O)CC([O-])=O)[C@H](O)C[C@H]1O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('OC1C(C(C(=O)C1)C/C=C/CCC(OC(C[N+](C)(C)C)CC([O-])=O)=O)/C=C/C(O)C/C=C\\\\C/C=C/CC', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), ('O([C@H]([N+](C)(C)C)CCC([O-])=O)C(=O)/C=C/CCCCC', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('C(CCC)C[C@@H](/C=C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCC(NCCCC([O-])=O)=O)OO', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('[C@H]1(N2C=CC(=O)NC2=O)[C@@H]([C@@H]([C@@]([C@H]([C@H]([NH3+])C(=O)[O-])O)(O1)[H])O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), ('CCCCCCCCC(=O)OC(CC([O-])=O)C[N+](C)(C)C', 'Contains "
               "carboxylate anion, hydroperoxy group, and 2 double bonds'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCC(=O)OC(CC([O-])=O)C[N+](C)(C)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), ('C([C@@H](CC([O-])=O)OC(=O)*C([O-])=O)[N+](C)(C)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('O[C@@H]1[C@@H](COC(=O)CC([O-])=O)O[C@@H](Oc2cc3c([O-])cc(O)cc3[o+]c2-c2cc(O)c(O)c(O)c2)[C@H](O)[C@H]1O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H](*)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O[C@@H]5O[C@@H]([C@@H]([C@@H]([C@H]5O)O[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(CO)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O[C@H]7[C@@H]([C@H]([C@@H](O)[C@H](O7)CO)O)NC(C)=O)CO)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('[C@@H]1([C@H](O[C@@H]([C@@H]([C@@H]1O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O)O)O)O)CO[C@@]3(C[C@@H]([C@H]([C@](O3)([H])[C@@H]([C@@H](CO)O)O)NC(=O)C)O)C([O-])=O)O*)NC(=O)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCCCCCCCCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('COC(=O)[C@]1(C[C@@H](O)[C@@H](O)[C@H](O1)[C@H](O)CO)OC[C@@H](O)[C@H]1O[C@@](C[C@@H](O)[C@H]1O)(OCC=C)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@H]1([C@@H](C[C@H]([C@@H](O1)C)O)O)OCCCCCCCCC\\\\C=C\\\\C(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('O([C@H]1[C@H]([C@@H]([C@@H]([C@@H](O1)C)O)O)O)[C@@H]2[C@H]([C@@H](O[C@@H]([C@H]2O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@@H]([C@H](O4)CO)O[C@H]5[C@H]([C@@H]([C@@H]([C@@H](O5)C)O)O)O)O[C@H]6[C@@H]([C@H]([C@H]([C@H](O6)CO)O)O[C@]7(O[C@@]([C@@H]([C@H](C7)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C(=O)[O-])O)NC(C)=O)O)CO)O[C@@H]8[C@H]([C@@H](O[C@@H]([C@@H]8O)CO)O[C@H]9[C@@H]([C@H]([C@@H](O[C@@H]9CO)OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)O)O)O)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[C@H]1(O[C@@H]([C@@H]([C@@H]([C@H]1O)O[C@]2(O[C@]([C@@H]([C@H](C2)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)O[C@H]3[C@@H]([C@H]([C@@H](O[C@@H]3CO)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)CO)O[C@@H]6O[C@@H]([C@@H]([C@@H]([C@H]6O)O)O)CO)O)NC(=O)C)O[C@H]7[C@@H]([C@H]([C@@H](O[C@@H]7CO)O[C@@H]8[C@H]([C@@H](O[C@@H]([C@@H]8O)CO)O[C@H]9[C@@H]([C@H]([C@@H](O[C@@H]9CO)OC[C@@H]([C@@H](CCCCCCCCCCCCCCC)O)NC(=O)*)O)O)O)NC(C)=O)O)O)NC(=O)C)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/CC\\\\C(C)=C/COP([O-])(=O)OP([O-])(=O)O[C@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O[C@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]3O[C@@H]3O[C@@H]([C@@H](O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]4O)[C@H](O)[C@H]3O)C([O-])=O)[C@H]2O)[C@H](O)[C@H]1O', "
               "'Contains carboxylate anion, hydroperoxy group, and 14 double "
               "bonds'), "
               "('C=1(C(CCC(C1C)=O)(C)C)/C=C/C(=C/C=C/C(=C/C(O[C@H]2[C@@H]([C@H]([C@@H]([C@H](O2)C(=O)[O-])O)O)O)=O)/C)/C', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('C[C@H](CCCC(=O)CC([O-])=O)O[C@@H]1O[C@@H](C)[C@H](O)C[C@H]1O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C1(/C(/NC([C@@H](N1)CCCNC(N)=[NH2+])=O)=C/C2=CC=C(C(=C2)OCC[C@H]([NH3+])C(=O)[O-])O)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[H+].[H+].[O-]C(=O)\\\\C=C/C([O-])=O.CN(CCOc1ccc(CC2SC(=O)NC2=O)cc1)c1ccccn1', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)COc1ccccc1)C([O-])=O.[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)COc1ccccc1)C([O-])=O.[H][N+]([H])(CC[N+]([H])([H])C[C@]1(C)CCC[C@]2(C)c3ccc(cc3CC[C@@]12[H])C(C)C)C[C@]1(C)CCC[C@]2(C)c3ccc(cc3CC[C@@]12[H])C(C)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('O([C@@H]1[C@H]([C@@H](O[C@@H]([C@H]1O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O)O[C@H]3[C@@H]([C@H]([C@@H]([C@H](O3)CO)O[C@@H]4O[C@@H]([C@@H]([C@@H]([C@H]4O)O[C@]5(O[C@]([C@@H]([C@H](C5)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)O)NC(=O)C)O)CO)O[C@@H]6[C@H]([C@@H](O[C@@H]([C@@H]6O)CO)O[C@H]7[C@@H]([C@H]([C@@H](O[C@@H]7CO)O[C@@H]8[C@H]([C@@H](O[C@@H]([C@@H]8O)CO)O[C@H]9[C@@H]([C@H]([C@@H](O[C@@H]9CO)OC[C@@H]([C@@H](*)O)NC(=O)*)O)O)O)NC(=O)C)O)O)NC(=O)C)[C@@H]%10O[C@H]([C@H]([C@H]([C@@H]%10O)O)O)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('[C@H]1([C@@H]([C@H](CO[C@H]1C/C(=C/C(OCCCCCCCC([O-])=O)=O)/C)C/C=C/[C@H]([C@H](C)O)C)O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('O[C@H]1[C@H]([C@H](O[C@H]([C@@H]1O)O[C@@H]2[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]2O)O)CO)CO)O[C@H]3[C@@H]([C@H]([C@@H](O)[C@H](O3)CO)O[C@H]4[C@@H]([C@@H](O[C@]5(O[C@]([C@@H]([C@H](C5)O)NC(C)=O)([C@@H]([C@H](O[C@]6(O[C@]([C@@H]([C@H](C6)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C(=O)[O-])CO)O)[H])C([O-])=O)[C@H]([C@H](O4)CO)O)O)NC(C)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('O([C@@H]1[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(*)=O)[C@@H]([C@H]1O)O)CO)[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@H]([C@H](O4)CO)O)O)O[C@@H]5O[C@H]([C@H]([C@H]([C@@H]5O)O)O)C)NC(C)=O)O[C@@]6(C[C@@H]([C@H]([C@@](O6)([C@@H]([C@@H](CO)O)O)[H])NC(CO)=O)O)C(=O)[O-])O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[C@H]1(O[C@H](C([O-])=O)[C@H]([C@@H]([C@H]1O)O)O)OC(\\\\C=C\\\\C=2C=CC(=C(C2)OC)O)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H](CCCCCCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO[C@]5(O[C@]([C@@H]([C@H](C5)O)NC(C)=O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O)O[C@@H]6O[C@@H]([C@@H]([C@@H]([C@H]6O)O[C@]7(O[C@]([C@@H]([C@H](C7)O)NC(C)=O)([C@@H]([C@@H](CO)O)O)[H])C([O-])=O)O)CO)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('C(=O)(OC[C@H](COP(OC[C@@H](C(=O)[O-])[NH3+])(=O)[O-])OC(=O)CCCCCCCC1C(CCCCCCCC)O1)CCCCCCCCCCCCCCCCC', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('[C@H](CCC([O-])=O)([N+](C)(C)C)OC(CC(CCCCCCC/C=C\\\\CCCCCC)O)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O[C@]5(O[C@]([C@@H]([C@H](C5)O)NC(C)=O)([C@@H]([C@@H](CO)O)OC(=O)C)[H])C([O-])=O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H](CCCCCCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O[C@@H]5O[C@@H]([C@@H]([C@@H]([C@H]5O)O)O[C@H]6[C@@H]([C@H]([C@@H](O)[C@H](O6)CO)O)NC(C)=O)CO)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('O([C@@H](C[N+](C)(C)C)CC([O-])=O)C(=O)CCCCCCCCCCCCCCCCCCCCCCC', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C([C@@H](CC([O-])=O)OC(CCCC)=O)[N+](C([2H])([2H])[2H])(C([2H])([2H])[2H])C([2H])([2H])[2H]', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C(C([O-])=O)C/C=C\\\\C\\\\C=C/C=C/[C@H](C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)OO', "
               "'Contains carboxylate anion, hydroperoxy group, and 7 double "
               "bonds'), "
               "('C1O[C@]([C@H]([C@@H]([C@@H]1O)O)O)(O)/C=N/CC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 2 double "
               "bonds'), "
               "('C12(C(CCCC1(C)O2)(C)C)/C=C/C(=C/C=C/C(=C/C([O-])=O)/C)/C', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), ('O([C@@H](C[N+](C)(C)C)CC([O-])=O)C(=O)CCC(O)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), ('C=1C=C(C=CC1C(=O)OCCO)C(=O)[O-]', 'Contains "
               "carboxylate anion, hydroperoxy group, and 2 double bonds'), "
               "('C1[C@H]2[C@@H]([C@H](O[C@@H]1C2)/C=C/[C@H](CCCCC)O)C/C=C\\\\CCCC(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('CC(=O)N[C@@H]1[C@@H](O)C[C@@](O[C@H](CO)[C@@H](O)[C@@H]2O[C@@](C[C@H](O)[C@H]2NC(C)=O)(O[C@@H]2[C@@H](O)[C@H](O[C@H]3[C@H](O)[C@@H](O)[C@H](OC[C@H](NC([*])=O)[C@H](O)[*])O[C@@H]3CO)O[C@H](CO)[C@@H]2O[C@@H]2O[C@H](CO)[C@H](O)[C@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O[C@@]4(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O4)[C@H](O)[C@H](O)CO)C([O-])=O)[C@H]3O)[C@H]2NC(C)=O)C([O-])=O)(O[C@H]1[C@H](O)[C@H](O)CO)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 8 double "
               "bonds'), "
               "('N1(C([C@H](C1)NC(/C(/C=2C=CC(OCC[C@@H](C(=O)[O-])[NH3+])=CC2)=N\\\\O)=O)=O)[C@@H](C(O)=O)C3=CC=C(C=C3)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('[C@H]1(CCCC([O-])=O)[C@H](/C=C/C=C/C=C\\\\C/C=C\\\\C=C\\\\C(CC)O)O1', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('C([C@@H]([C@@H](CCCCCCCCCCCCCCCCC)O)NC(*)=O)O[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)CO)O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@H]([C@H](O4)CO)O)O)O)NC(C)=O)O[C@@]5(C[C@@H]([C@H]([C@@](O5)([C@@H]([C@@H](CO)O)O)[H])NC(CO)=O)O)C([O-])=O)O)O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('C[C@H](NC(=O)[C@@H](C)O[C@H]1[C@H](O)[C@@H](CO)OC(OP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2O)n2ccc(=O)[nH]c2=O)[C@@H]1NC(C)=O)C(=O)N[C@H](CCC(=O)N[C@@H](CCC[C@@H]([NH3+])C([O-])=O)C([O-])=O)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 11 double "
               "bonds'), "
               "('[H+].[H+].[O-]C(=O)\\\\C=C/C([O-])=O.CCCCCNC(=N)N\\\\N=C\\\\c1c[nH]c2ccc(OC)cc12', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('C1=CC(=NC(N1[C@@H]2O[C@H](COP(O[C@]3(C[C@H](O)[C@@H](O)[C@@](O3)([C@@H]([C@@H](CO)O)O)[H])C(=O)[O-])(=O)[O-])[C@H]([C@H]2O)O)=O)N', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@H]1(O[C@@H](O[C@@H]2[C@@H](NC(C)=O)[C@@H](O[C@H](CO)[C@H]2O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O[C@]4(O[C@@]([C@@H]([C@H](C4)O)NC(C)=O)([H])[C@@H]([C@@H](CO)O)O)C([O-])=O)[C@H]3O)O*)[C@H]([C@@H]([C@@H]1O)O)O)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), ('C\\\\C=C(/C)C(=O)O[C@H](CC([O-])=O)C[N+](C)(C)C', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('C(C([O-])=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC1C(CC)O1', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('C(CCCCCC/C=C/[C@@H](CC1C(C/C=C\\\\CC)O1)O)(=O)[O-]', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('O1/C(/C[C@]2([C@@]1(C[C@H]([C@@H]2/C=C/[C@H](CCCC(C)O)O)O)[H])[H])=C\\\\CCCC([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 3 double "
               "bonds'), "
               "('[C@@]1(O[C@H]2[C@H]([C@H](O[C@H]([C@@H]2O)O[C@@H]3[C@H](O[C@@H](OC[C@@H]([C@@H]([C@@H](CCCCCCCCCCCCCC)O)O)NC(=O)*)[C@@H]([C@H]3O)O)CO)CO)O[C@H]4[C@@H]([C@H]([C@@H](O)[C@H](O4)CO)O[C@@H]5O[C@@H]([C@@H]([C@@H]([C@H]5O)O)O[C@H]6[C@@H]([C@H]([C@@H](O)[C@H](O6)CO)O)NC(C)=O)CO)NC(C)=O)(O[C@]([C@H](NC(=O)C)[C@H](C1)O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 5 double "
               "bonds'), "
               "('C([C@@H]([C@@H](CCCCCCCCCCCCCCCCC)O)NC(*)=O)O[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)CO)O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O[C@H]3[C@@H]([C@H]([C@H]([C@H](O3)CO)O)O[C@H]4[C@@H]([C@H]([C@H]([C@H](O4)CO)O)O)O)NC(C)=O)O[C@@]5(C[C@@H]([C@H]([C@@](O5)([C@@H]([C@@H](CO)O)O)[H])NC(C)=O)O)C([O-])=O)O)O)O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('[Na+].[H][C@]12SCC(COC(C)=O)=C(N1C(=O)[C@H]2NC(=O)C(=N/OC)\\\\c1csc(N)n1)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds'), "
               "('OC[C@H]1C(OC[C@@H](C([O-])=O)NC(=O)C2=C3C(=CC(=C2)[C@H]4[C@@H]([C@H]([C@@H]([C@H](O4)CO)O)O)O)O[Fe-]5(OC6=C(C=C(C=C6O5)[C@H]7[C@@H]([C@H]([C@@H]([C@H](O7)CO)O)O)O)C(=O)N1)O3)=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 4 double "
               "bonds'), "
               "('CCCC[C@@H](C)[C@@H](OC(=O)C[C@@H](CC([O-])=O)C([O-])=O)[C@H](C[C@@H](C)C[C@H](O)CCCC[C@@H](O)C[C@H](O)[C@H](C)[NH3+])OC(=O)C[C@@H](CC([O-])=O)C([O-])=O', "
               "'Contains carboxylate anion, hydroperoxy group, and 6 double "
               "bonds')]\n"
               'False negatives: []',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 21,
    'num_true_negatives': 183829,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.27586206896551724,
    'recall': 1.0,
    'f1': 0.4324324324324324,
    'accuracy': 0.9998857814182684}


"""
Classifies: CHEBI:33521 metal atom
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_metal_atom(smiles: str):
    """
    Determines if the given SMILES string represents a metal atom.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a metal atom, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    if mol.GetNumAtoms() != 1:
        return False, "Not a single atom"

    atom = mol.GetAtomWithIdx(0)
    symbol = atom.GetSymbol()
    atomic_number = atom.GetAtomicNum()

    # List of metal elements based on the IUPAC definition
    metal_elements = [
        'Li', 'Be', 'Na', 'Mg', 'Al', 'K', 'Ca', 'Sc', 'Ti', 'V', 'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn',
        'Ga', 'Rb', 'Sr', 'Y', 'Zr', 'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Cs', 'Ba',
        'La', 'Ce', 'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu', 'Hf', 'Ta',
        'W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 'Bi', 'Po', 'Fr', 'Ra', 'Ac', 'Th', 'Pa', 'U',
        'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 'Es', 'Fm', 'Md', 'No', 'Lr', 'Rf', 'Db', 'Sg', 'Bh', 'Hs', 'Mt',
        'Ds', 'Rg', 'Cn', 'Nh', 'Fl', 'Mc', 'Lv', 'Ts', 'Og'
    ]

    if symbol in metal_elements:
        return True, f"{symbol} is a metal atom"
    else:
        return False, f"{symbol} is not a metal atom"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33521',
                          'name': 'metal atom',
                          'definition': 'An atom of an element that exhibits '
                                        'typical metallic properties, being '
                                        'typically shiny, with high electrical '
                                        'and thermal conductivity.',
                          'parents': ['CHEBI:33250']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 15,
    'num_false_positives': 100,
    'num_true_negatives': 109477,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.13043478260869565,
    'recall': 1.0,
    'f1': 0.23076923076923078,
    'accuracy': 0.9990875246368348}


"""
Classifies: CHEBI:26207 polyterpenoid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_polyterpenoid(smiles: str):
    """
    Determines if a molecule is a polyterpenoid (C5n skeleton where n > 8).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a polyterpenoid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Get number of carbons
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    
    # Check if number of carbons is divisible by 5 (C5n)
    if num_carbons % 5 != 0:
        return False, f"Number of carbons ({num_carbons}) is not divisible by 5"
        
    # Calculate n from C5n
    n = num_carbons / 5
    
    # Check if n > 8 for polyterpenoid
    if n <= 8:
        return False, f"n = {n} is not greater than 8 (C{num_carbons} skeleton)"
        
    # Check for presence of isoprene units (branched 5-carbon units)
    # Look for methyl branches which are characteristic of terpenes
    methyl_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            # Count carbons with single methyl group
            if len([n for n in atom.GetNeighbors() if n.GetSymbol() == 'C' and n.GetDegree() == 1]) == 1:
                methyl_count += 1
                
    if methyl_count < n: # Should have roughly one methyl per isoprene unit
        return False, f"Insufficient methyl branches ({methyl_count}) for a terpenoid structure"
    
    # Additional check for presence of oxygen (most terpenoids are oxygenated)
    has_oxygen = any(atom.GetSymbol() == 'O' for atom in mol.GetAtoms())
    
    return True, f"Polyterpenoid with C{num_carbons} skeleton (n={n}){' containing oxygen' if has_oxygen else ''}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26207',
                          'name': 'polyterpenoid',
                          'definition': 'A polymeric terpenoid having a C5n '
                                        'skeleton, where n is greater than 8.',
                          'parents': ['CHEBI:26873']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 55,
    'num_true_negatives': 183847,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.03508771929824561,
    'recall': 0.6666666666666666,
    'f1': 0.06666666666666667,
    'accuracy': 0.9996954949566352}


"""
Classifies: CHEBI:24745 hydroxypyridine
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxypyridine(smiles: str):
    """
    Determines if a molecule is a hydroxypyridine (pyridine with at least one hydroxy substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxypyridine, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for pyridine ring with hydroxy substituent
    patterns = [
        # Basic pyridine with OH
        Chem.MolFromSmarts('c1ccncc1[OH]'),
        # Alternative forms of pyridine with OH
        Chem.MolFromSmarts('[nH]1c(O)cccc1'),
        Chem.MolFromSmarts('n1c(O)cccc1'),
        # Keto form
        Chem.MolFromSmarts('O=c1[nH]cccc1'),
        # N-oxide form
        Chem.MolFromSmarts('[O-][n+]1ccccc1'),
        # Various tautomeric forms
        Chem.MolFromSmarts('Oc1[nH]cccc1'),
        Chem.MolFromSmarts('Oc1ccncc1'),
        Chem.MolFromSmarts('O=c1cccc[nH]1'),
    ]

    found = False
    for pattern in patterns:
        if pattern is not None and mol.HasSubstructMatch(pattern):
            found = True
            break

    if not found:
        return False, "No hydroxy substituent found on pyridine ring"

    # Find position of hydroxy group(s)
    positions = []
    for pattern in patterns:
        if pattern is None:
            continue
        matches = mol.GetSubstructMatches(pattern)
        for match in matches:
            # For each match, find the position relative to nitrogen
            for ring in mol.GetRingInfo().AtomRings():
                if any(idx in match for idx in ring):
                    # Find nitrogen and oxygen indices in the ring
                    n_idx = None
                    o_idx = None
                    for idx in ring:
                        atom = mol.GetAtomWithIdx(idx)
                        if atom.GetSymbol() == 'N':
                            n_idx = idx
                        elif atom.GetSymbol() == 'O':
                            neighbors = [n.GetIdx() for n in atom.GetNeighbors()]
                            if any(nidx in ring for nidx in neighbors):
                                o_idx = idx

                    if n_idx is not None and o_idx is not None:
                        # Calculate position
                        for neighbor in mol.GetAtomWithIdx(o_idx).GetNeighbors():
                            if neighbor.GetIdx() in ring:
                                pos = (ring.index(neighbor.GetIdx()) - ring.index(n_idx)) % 6
                                if pos == 0:
                                    positions.append(1)  # N-hydroxy
                                else:
                                    positions.append(pos)

    positions = list(set(positions))  # Remove duplicates
    positions.sort()

    if len(positions) == 1:
        return True, f"{positions[0]}-hydroxypyridine"
    elif len(positions) > 1:
        return True, f"Multiple hydroxypyridine positions: {','.join(map(str, positions))}"
    else:
        return True, "Hydroxypyridine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24745',
                          'name': 'hydroxypyridine',
                          'definition': 'Any member of the class of pyridines '
                                        'with at least one hydroxy '
                                        'substituent.',
                          'parents': ['CHEBI:26421', 'CHEBI:74818']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.10619469026548672 is too low.\n'
               "True positives: [('NCCCC(=O)C1=CN=C(O)C=C1', "
               "'5-hydroxypyridine'), "
               "('C(C)(O)=O.C=1(N=C(C(O)=CC1)CO)C(CNC(C)(C)C)O', "
               "'4-hydroxypyridine'), ('Cc1ncc(cc1O)C(O)=O', "
               "'2-hydroxypyridine'), ('O/N=C\\\\C1=C(O)C(=NC=C1CO)C', "
               "'2-hydroxypyridine'), ('OC1=C(C(=CN=C1C)CO)C2SCC(N2)C(=O)O', "
               "'4-hydroxypyridine'), "
               "('CC(C)CC(=O)OC(c1ccccc1)c1cc(O)c(C(O)=O)c(=O)[nH]1', "
               "'3-hydroxypyridine')]\n"
               'False positives: '
               "[('O=C1N(C=C([C@]2(O)[C@H](O)CC(=O)CC2)C(=C1[C@H]3O[C@@H](/C(=C/C(CC(CC)C)C)/C)[C@H](C)CC3)O)C', "
               "'3-hydroxypyridine'), "
               "('O=C1N(O)C=C(C2=CC=C(O[C@H]3OC([C@H](OC)[C@@H](C3O)O)CO)C=C2)C(=C1C4OC(CCC4C)C)O', "
               "'Hydroxypyridine'), "
               "('O1C(C2OC3=C(C2O)C=4N(C=5C(C(=O)C4C(O)=C3)=CC=CC5)C)(C1)C', "
               "'Hydroxypyridine'), "
               "('CCCCCN1C2=CC=CC=C2C(=C(C1=O)C(=O)NC(C)C3=CC=CC=C3)O', "
               "'3-hydroxypyridine'), "
               "('O1C(OC2=C(O)C(=[N+]([O-])C=C2)C3=[N+]([O-])C=CC(=C3O)OC4OC(C(O)C(C4O)O)CO)C(O)C(O)C(C1CO)O', "
               "'4-hydroxypyridine'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C(=C1C(=O)[C@@H]3C=C([C@H]4CC[C@@H](C[C@@H]4[C@H]3/C=C/CO)C)C)O', "
               "'3-hydroxypyridine'), ('OC(=O)[C@@H](N)CC1=NC=C(O)C=C1', "
               "'4-hydroxypyridine'), "
               "('CC(C)CCn1c2ccccc2c(O)c(C(=O)Nc2ncc(C)s2)c1=O', "
               "'3-hydroxypyridine'), "
               "('C1CCC2=C(C1)C3=C4N2C(=O)C(=C(C4=CC=C3)O)CC5=CC=CC=C5', "
               "'3-hydroxypyridine'), ('CC(=NC1=CC=CC=C1O)CC(=O)C2=CC=NC=C2', "
               "'Hydroxypyridine'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C3=C1C(=O)[C@H]4[C@H](C=CC)[C@H]5C[C@@H](C)CC[C@@H]5[C@@]([C@@H]4O3)(O)C', "
               "'Hydroxypyridine'), ('O=C1C(O)=C2N=CC3=C2C(=C1)N(C=C3)CCCO', "
               "'Hydroxypyridine'), "
               "('CC[C@H]1NC(=O)[C@@H](NC(=O)c2ncccc2O)[C@@H](C)OC(=O)[C@@H](NC(=O)C2CC(=O)C(CS[C@@H]3CN4CCC3CC4)CN2C(=O)[C@H](Cc2ccc(cc2)N(C)C)N(C)C(=O)[C@@H]2CCCN2C1=O)c1ccccc1', "
               "'4-hydroxypyridine'), ('CCCOC(=O)CC1=C(C2=CC=CC=C2NC1=O)O', "
               "'3-hydroxypyridine'), "
               "('O=C1NC=C([C@]2(O)[C@@H]3O[C@@H]3[C@H](O)CC2)C(=C1C(=O)[C@H]4[C@H]5[C@H](C=C[C@H]4C)[C@@H](O)[C@H](C)CC5)O', "
               "'3-hydroxypyridine'), ('Oc1cccc2[nH]c(=O)ccc12', "
               "'Hydroxypyridine'), "
               "('COC(=O)\\\\C=C/NC(=O)c1cc2c3cccc(O)c3[nH]c2c(n1)C(C)=O', "
               "'Hydroxypyridine'), "
               "('O1C(CC(C2=C1C(C(C)(C)C=C)=C3OC(=O)C(=CC3=C2O)C(C)(C)C=C)C=4C=5NC6=C(C(=O)C5C(O)=CC4OC)C=CC(O)=C6OC)(C)C', "
               "'Hydroxypyridine'), "
               "('[S@](=O)(C=1C=2NC=C(O)C=3C2N(C4=CC(=O)C=CC34)C(C1)=O)C', "
               "'2-hydroxypyridine'), "
               "('O1C(C1C(O)C=2C=3N(C4=C(C(=O)C3C(O)=CC2OC)C=CC(OC)=C4OC)C)(C)C', "
               "'Hydroxypyridine'), "
               "('C1CN2C3=C(C=CC=C31)C(=C(C2=O)C(=O)NCCC4=CC=CC=C4)O', "
               "'3-hydroxypyridine'), ('CCCCCCCc1cc(O)c2ccccc2[n+]1[O-]', "
               "'3-hydroxypyridine'), "
               "('CN1C(=C(C(=O)NC1=S)C(C2=CC=CC=N2)C3=C(N(C(=S)NC3=O)C)O)O', "
               "'5-hydroxypyridine'), ('OC(=O)c1c(O)[nH]c2ccccc2c1=O', "
               "'1-hydroxypyridine'), "
               "('O=C(O)C1=NC2=C(O)C(O)=C3O[C@]4(CC[C@H]5[C@]([C@@H]4CC3=C2C=C1)(CCCC5(C)C)C)C', "
               "'Hydroxypyridine'), "
               "('O=C1C=2N=CC3=CC(CC)=CC(=C3C2C(=O)C=4C1=C(O)C=CC4)O', "
               "'Hydroxypyridine'), ('O=C1NC(O)=CC2=NN(C(N)=C21)C3=CC=CC=C3', "
               "'5-hydroxypyridine'), "
               "('O=C1N(C=C(C2(O)CCC(=O)CC2)C(=C1[C@H]3O[C@@H](/C(=C/C(CC(CC)C)C)/C)[C@H](C)CC3)O)C', "
               "'3-hydroxypyridine'), "
               "('CCN(C1CCOCC1)C2=CC(=CC(=C2C)/C(=N/CC=3C(C)=CC(C)=NC3O)/O)C4=CC=C(C=C4)CN5CCOCC5', "
               "'1-hydroxypyridine'), "
               "('CCCCn1c2ccccc2c(O)c(C(=O)Nc2nnc(CC)s2)c1=O', "
               "'3-hydroxypyridine'), ('O=C(O)C=1C2=NC=CC=C2C=C(C1)O', "
               "'Hydroxypyridine'), "
               "('S1C2=NC(=C1)C(=O)N[C@H](C=3SC=C(N3)C(=O)N[C@H](C=4SC=C(C5=C(C6=NC(C(N[C@H](C(NC2=C(OC)C)=O)[C@H](O)C)=O)=CS6)C=C(O)C(=N5)C=7SC=C(N7)C(=O)NC(C(=O)N)=C)N4)CO)[C@H](O)[C@H](O)C(=O)O', "
               "'4-hydroxypyridine'), "
               "('C[C@H](OP(O)(=O)OC[C@@H](O)[C@@H](O)[C@@H](O)Cn1c2cc(O)ccc2cc2c1nc(=O)[nH]c2=O)C(=O)N[C@@H](CCC(=O)N[C@@H](CCC(=O)N[C@@H](CCC(O)=O)C(O)=O)C(O)=O)C(O)=O', "
               "'Hydroxypyridine'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C(=C1C(=O)C3=C(/C=C/C)[C@@H]4[C@H](CC[C@@H](C4)C)C(=C3)C)O', "
               "'3-hydroxypyridine'), "
               "('ClC(C(CC(C(=O)C=1C(=O)NC(OC)=C(C1O)OC)C)C)CCl', "
               "'3-hydroxypyridine'), ('O=C1NC(=CC(=C1C=O)O)CC2=CC=CC=C2', "
               "'3-hydroxypyridine'), "
               "('O=C1N(O)C=C([C@@]2(O)[C@H]3O[C@H]3[C@@H](O)CC2)C(=C1C(=O)[C@@H]4[C@@H]5[C@@H](C=C[C@@H]4C)CC(C)=CC5)O', "
               "'Hydroxypyridine'), "
               "('S1(=O)(=O)N(C(=C(O)C=2C1=CC=CC2)C(=O)NC3=NC=C(O)C=C3)C', "
               "'2-hydroxypyridine'), "
               "('O=C1O[C@@H]([C@H](NC(=O)C2=NC=CC=C2O)C(=O)N[C@@H](C(=O)N3[C@@H](C(=O)N(C)CC(N([C@H](C(N[C@H](C(N([C@H]1C4=CC=CC=C4)C)=O)CO)=O)C(C(C)C)C)C)=O)C[C@H](C3)O)CC(C)C)C', "
               "'4-hydroxypyridine'), "
               "('ClC1=C(OC)C=2OC3=C4OCO[C@H]5C4=C(C=6C(=O)C=7C(=O)NC(C)=CC7C(C6C5)=O)C(=C3C(C2C=C1)=O)O', "
               "'Hydroxypyridine'), "
               "('COc1ccc(cc1O)-c1c2c3cc(OC)c(OS(O)(=O)=O)cc3oc(=O)c2n2ccc3cc(OC)c(OC)cc3c12', "
               "'Hydroxypyridine'), "
               "('O1C(C=CC2=C1C=C(O)C=3C2=NC=4C(C3O)=CC=CC4)(C)C', "
               "'3-hydroxypyridine'), "
               "('CCN1C2=CC=CC=C2C(=C(C1=O)C(=O)NC3=CC=CC=C3F)O', "
               "'3-hydroxypyridine'), "
               "('COc1ccc(cc1O)-c1c2c3cc(OC)c(OS([O-])(=O)=O)cc3oc(=O)c2n2ccc3cc(OC)c(OC)cc3c12', "
               "'Hydroxypyridine'), "
               "('OC(=O)c1cc(C(O)=O)c2c(n1)c(O)c(O)c1cc([nH]c21)C(O)=O', "
               "'Hydroxypyridine'), "
               "('O=C1N(O)CCCC1NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)[C@H]2NC3=[N+](C=4C(=CC(O)=C(C4)O)C=C3NC(=O)CCC(N)C(=O)O)CC2)CC(=O)O)CCCCN)C(O)C)C(O)C(=O)O)C(O)C)C(O)C', "
               "'Hydroxypyridine'), ('Cc1ncc(CO)c(C[NH3+])c1O', "
               "'2-hydroxypyridine'), "
               "('C12=CC(=CC=C1C(=CC=N2)NC3=CC(=C(C=C3)O)C[NH+](CC)CC)Cl', "
               "'Hydroxypyridine'), "
               "('O=C(O)C1=C(C(O)=C(O)C(=C1C)O)C2=NC=CC3=C2N(C=4C=CC=CC34)C', "
               "'Hydroxypyridine'), ('COc1cc2n(C)c3c(O)cccc3c(=O)c2c(O)c1OC', "
               "'Hydroxypyridine'), ('C1(O)=C(C)[NH+]=CC(CO)=C1C[NH3+]', "
               "'2-hydroxypyridine'), "
               "('ClC1=C(O)C2=C(O)C3=C(C(=O)C4(O)C5=C(OC)C=6C(=O)NC(C)=CC6C=C5CCC4C3=O)C(=C2C=C1O)C', "
               "'Hydroxypyridine'), "
               "('S1SCC2N(C(=O)CNC(=O)C(NC(=O)C3=NC=4C(=CC=CC4)C=C3O)CSC(C(N(C(C(C1)N(C(=O)CNC(=O)C(NC(=O)C5=NC=6C(=CC=CC6)C=C5O)CSC(C(N(C2=O)C)=C)=O)C)=O)C)=C)=O)C', "
               "'4-hydroxypyridine'), "
               "('C1=C(C(=C(C=C1C2=NC(=NO2)C=3C(=[N+](C(=C(C3C)Cl)C)[O-])Cl)O)O)[N+](=O)[O-]', "
               "'Hydroxypyridine'), "
               "('O=C1C(O)=C2N=C3C(=CC(=O)C=C3)C4=C2C(=C1)C=CN4', "
               "'Hydroxypyridine'), "
               "('O=C1OC(C(NC(=O)C2=NC=CC=C2O)C(=O)NC(C(=O)N3C(C(=O)N(C)CC(NC(C(NC(C(N(C1C4=CC=CC=C4)C)=O)C)=O)C(C(C)C)C)=O)C[C@H](C3)O)C(CC)C)C', "
               "'4-hydroxypyridine'), ('Oc1ccc2nc(Cl)ccc2c1', "
               "'Hydroxypyridine'), "
               "('S(=O)(=O)(N1CCN(C(=O)C=2C=3C(N=C(O)C2)=CC=CC3)CC1)C=4C(=C(C=C(C4C)C)C)C', "
               "'1-hydroxypyridine'), "
               "('CC(C)Cn1c2ccccc2c(O)c(C(=O)Nc2ncc(C)s2)c1=O', "
               "'3-hydroxypyridine'), "
               "('C(NN=CC1=CC(OC)=C(C=C1)O)(=O)C=2C=CN=CC2', "
               "'Hydroxypyridine'), "
               "('O=C1N2C=3C(O)=C(NC(=O)[C@@H](O)C)C4=CC=C(N=C4C3C=C[C@@]2(C)CC1)C', "
               "'Hydroxypyridine'), ('OC1=C(C(=O)N(C=2C1=CC=CC2)C)CC', "
               "'3-hydroxypyridine'), "
               "('N=1C=CC=C2C1OC=3C2=NC(=NC3N4CCOCC4)C=5C=CC=C(O)C5', "
               "'Hydroxypyridine'), "
               "('O=C1NC(C(=O)NC(C(=O)NCCCCC1NC(=O)C(NC(=O)CNC(=O)C(NC(=O)C(NC(=O)C2[N+]3=C(NCC2)C(NC(=O)CCC(=O)O)=CC=4C3=CC(O)=C(C4)O)CO)CCCCN)CCCN(O)C=O)CO)CCCN(O)C=O', "
               "'Hydroxypyridine'), "
               "('COC1=C(C=CC(=C1)C=C2C(=O)N=C(S2)NC3=CC=CC=N3)O', "
               "'Hydroxypyridine'), "
               "('CC(C)Cn1c2ccccc2c(O)c(C(=O)Nc2nc(C)cs2)c1=O', "
               "'3-hydroxypyridine'), ('C[NH+]1CCC=C1c1ccc(O)nc1', "
               "'5-hydroxypyridine'), "
               "('COC1=CC=C(C=C1)C(=O)NC(C2=CC(=C(C=C2)OC)OC)C3=CC(=C4C=CC=NC4=C3O)Cl', "
               "'Hydroxypyridine'), "
               "('O[C@@H](CNC1CC=2C(C1)=CC(=C(C2)CC)CC)C3=C4C(NC(=O)C=C4)=C(O)C=C3.OC(=O)/C=C/C(O)=O', "
               "'Hydroxypyridine'), ('Oc1ccc-2c(c1)-c1nccc3ccnc-2c13', "
               "'Hydroxypyridine'), "
               "('CCC(CO)\\\\C=C\\\\C=C\\\\C=C\\\\C(=O)c1c(O)c(c[nH]c1=O)[C@@]1(O)CC[C@H](O)CC1', "
               "'3-hydroxypyridine'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C3=C1[C@H]4[C@@H](C[C@H](C)[C@H]([C@@H]4C)O)[C@@H](O3)C', "
               "'Hydroxypyridine'), ('CC1=CC(=O)NC2=C1C=CC(=C2)O', "
               "'Hydroxypyridine'), "
               "('O(C1=C(C(C2=C(OC)C=C3OC(=O)C=CC3=C2)C=C(C)C)C(O)=C4C(N(C5=C(C4=O)C=CC(OC)=C5OC)C)=C1)C', "
               "'Hydroxypyridine'), "
               "('[H+].[H+].O.O.[Cl-].[Cl-].CCN(CC)Cc1cc(Nc2ccnc3cc(Cl)ccc23)ccc1O', "
               "'Hydroxypyridine'), "
               "('CC1CC11N(C)C(=O)C2CSSCC(N(C)C(=O)C(C)NC(=O)C(COC1=O)NC(=O)c1nc3ccccc3cc1O)C(=O)N(C)C1(CC1C)C(=O)OCC(NC(=O)c1nc3ccccc3cc1O)C(=O)NC(C)C(=O)N2C', "
               "'4-hydroxypyridine'), ('O(C(=O)C=1C2=C(NC(=O)C1)C=CC(O)=C2)C', "
               "'Hydroxypyridine'), ('O=C(O)C1=C2C3=NC(=CC=C3COC2=CC(=C1)O)C', "
               "'Hydroxypyridine'), "
               "('O=C1C=2N=CC3=CC(C)=CC(=C3C2C(=O)C=4C1=C(O[C@@H]5O[C@H]([C@H](O)[C@@H](C5)O)C)C=CC4)O', "
               "'Hydroxypyridine'), "
               "('O=C1NC(=C(CCC)C(=C1C)O)C/C=C(/C/C=C/C(O)(C)C)\\\\C', "
               "'3-hydroxypyridine'), "
               "('O=C1N(C=C(C2(O)C(O)CC(=O)CC2)C(=C1C3OC(C(=CC(CC(CC)C)C)C)C(C)CC3)O)C', "
               "'3-hydroxypyridine'), "
               "('COC1=C(C=C(C=C1)C=NNC(=O)C2=C(C=C(C=C2)O)O)CSC3=CC=CC=N3', "
               "'Hydroxypyridine'), "
               "('CCCN1C2=CC=CC=C2C(=C(C1=O)C(=O)NCC3=CC(=C(C=C3)OC)OC)O', "
               "'3-hydroxypyridine'), "
               "('CCCN1C2=CC=CC=C2C(=C(C1=O)C(=O)NCC3=CC=NC=C3)O', "
               "'3-hydroxypyridine'), ('O=C1NC=C(C2=CC=CC=C2)C(=C1C=O)O', "
               "'3-hydroxypyridine'), "
               "('CC1=C(C(=O)OC2=CC(=C(C=C12)Cl)O)CC(=O)N3C[C@H]4C[C@@H](C3)C5=CC=CC(=O)N5C4', "
               "'Hydroxypyridine'), "
               "('O=C1N(OC)C=C(C2=CC=CC=C2)C(=C1[C@@H]3O[C@@H]([C@@H](C)C[C@H]3C)[C@@H](CCOC(=O)C)C)O', "
               "'3-hydroxypyridine'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C(=C1C(=O)[C@H]3[C@H]4[C@H](C=C[C@H]3C)CC(C)=CC4)O', "
               "'3-hydroxypyridine'), "
               "('O=C/1C2=C(C=NC(=C2)C(=O)N)C\\\\C1=C/C3=CC=C(O)C=C3', "
               "'Hydroxypyridine'), ('CC1=NC2=C(C=C(C=C2)O)C(=C1)NCCO', "
               "'Hydroxypyridine'), "
               "('CCN1CCN(CC1)C(C2=CN=CC=C2)C3=C(C4=C(C=CC=N4)C=C3)O', "
               "'Hydroxypyridine'), ('[NH3+]CCCC(=O)C1=CN=C(O)C=C1', "
               "'5-hydroxypyridine'), "
               "('O=C(OC)C1=NC(=C2NC3=C(C2=C1)C=C(O)C=C3)[C@@H](O)C', "
               "'Hydroxypyridine'), "
               "('O1C(CC(C2=C1C(C(C)(C)C=C)=C3OC(=O)C(=CC3=C2O)C(C)(C)C=C)C=4C=5NC6=C(C(=O)C5C(O)=CC4OC)C=CC(OC)=C6OC)(C)C', "
               "'Hydroxypyridine'), "
               "('O=C1N(OC)C=C(C2=CC=CC=C2)C(=C1[C@@H]3O[C@@H]([C@@H](C)C[C@H]3C)[C@H](CO)CC)O', "
               "'3-hydroxypyridine'), "
               "('C[C@H]1CCc2c1c[n+](CCc1ccc(O)cc1)cc2C', 'Hydroxypyridine'), "
               "('S1C(=NC(=C1)C2=NC(=C(O)C(=C2OC)OC)C(=O)N)CCCC(=O)C', "
               "'2-hydroxypyridine'), "
               "('CC(C)(C)N1C(=NN=N1)C(C2=CC=NC=C2)N3CCN(CC3)C4=CC=C(C=C4)O', "
               "'Hydroxypyridine'), "
               "('O=C1N(C=C(N1)C=O)[C@@H]2O[C@@H]([C@@H](NC(=O)[C@H](N)C[C@H](O)C3=NC=C(O)C=C3)C(=O)O)[C@@H]([C@@H]2O)O', "
               "'2-hydroxypyridine'), "
               "('O=C1N(O)C=C(C2=CC=C(O)C=C2)C(=C1C(=O)/C=C/C=C/C=C/C(=C/[C@H](CC)C)/C)O', "
               "'Hydroxypyridine')]\n"
               'False negatives: '
               "[('O=C1NC2=C([C@]3(N)/C(/[C@H](C2)C=C(C3)C)=C\\\\C)C=C1', 'No "
               "hydroxy substituent found on pyridine ring')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 6579,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9850433742147772}


"""
Classifies: CHEBI:26208 polyunsaturated fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_polyunsaturated_fatty_acid(smiles: str):
    """
    Determines if a molecule is a polyunsaturated fatty acid (PUFA).
    A PUFA contains a carboxylic acid group and multiple carbon-carbon double bonds.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a PUFA, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "No carboxylic acid group found"

    # Count carbon-carbon double bonds
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
    
    if double_bonds < 2:
        return False, f"Only {double_bonds} carbon-carbon double bonds found (needs 2+ for PUFA)"

    # Check carbon chain length (should be at least 12 carbons for typical PUFAs)
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    
    # Calculate the number of rings
    ring_info = mol.GetRingInfo()
    ring_count = ring_info.NumRings()
    
    # Additional information about double bond configuration
    cis_double_bonds = len(mol.GetSubstructMatches(Chem.MolFromSmarts('C/C=C\C')))
    trans_double_bonds = len(mol.GetSubstructMatches(Chem.MolFromSmarts('C/C=C/C')))
    
    description = f"PUFA with {double_bonds} double bonds ({cis_double_bonds} cis, {trans_double_bonds} trans), "
    description += f"{carbon_count} carbons"
    if ring_count > 0:
        description += f", {ring_count} rings"

    return True, description


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26208',
                          'name': 'polyunsaturated fatty acid',
                          'definition': 'Any fatty acid containing more than '
                                        'one double bond. Acids in this group '
                                        'are reported to have cardioprotective '
                                        'effects; and levels are lowered in '
                                        'chronic fatigue syndrome.',
                          'parents': ['CHEBI:27208']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 75,
    'num_false_positives': 100,
    'num_true_negatives': 3741,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.42857142857142855,
    'recall': 0.9615384615384616,
    'f1': 0.5928853754940712,
    'accuracy': 0.9737177851492728}


"""
Classifies: CHEBI:10036 wax ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_wax_ester(smiles: str):
    """
    Determines if a molecule is a wax ester - a fatty acid ester formed between a fatty acid
    and a fatty alcohol.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a wax ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for ester group (R-O-C(=O)-R)
    ester_pattern = Chem.MolFromSmarts('[CH2,CH3]-[O;H0]-C(=O)-[C;!R]')
    if not mol.HasSubstructMatch(ester_pattern):
        return False, "No ester group found"

    # Get the ester atoms
    ester_matches = mol.GetSubstructMatches(ester_pattern)
    if not ester_matches:
        return False, "No valid ester group found"
    
    # Calculate ring information
    ring_info = mol.GetRingInfo()
    ring_atoms = set()
    for ring in ring_info.AtomRings():
        ring_atoms.update(ring)

    # For each ester group found
    for match in ester_matches:
        # Skip if ester is part of a lactone (cyclic ester)
        if any(idx in ring_atoms for idx in match):
            continue

        # Get the atoms on either side of the ester
        alcohol_carbon = mol.GetAtomWithIdx(match[0])
        acid_carbon = mol.GetAtomWithIdx(match[3])
        
        # Check alcohol side chain length (should be at least 8 carbons for fatty alcohol)
        alcohol_chain = get_chain_length(mol, alcohol_carbon, ring_atoms)
        if alcohol_chain < 8:
            continue
            
        # Check acid side chain length (should be at least 8 carbons for fatty acid)
        acid_chain = get_chain_length(mol, acid_carbon, ring_atoms)
        if acid_chain < 8:
            continue
            
        # Check if the molecule has too many heteroatoms
        num_n = len(mol.GetSubstructMatches(Chem.MolFromSmarts('[N]')))
        num_o = len(mol.GetSubstructMatches(Chem.MolFromSmarts('[O]')))
        num_other = len(mol.GetSubstructMatches(Chem.MolFromSmarts('[!#1!#6!#7!#8]')))
        
        if num_n > 2 or num_o > 3 or num_other > 0:
            continue

        return True, f"Wax ester found with alcohol chain length {alcohol_chain} and acid chain length {acid_chain}"

    return False, "No fatty acid ester found with sufficient chain lengths"

def get_chain_length(mol, start_atom, ring_atoms):
    """Helper function to get the length of a carbon chain from a starting atom"""
    visited = set()
    queue = [(start_atom, 1)]
    max_length = 1
    
    while queue:
        current_atom, length = queue.pop(0)
        visited.add(current_atom.GetIdx())
        
        for neighbor in current_atom.GetNeighbors():
            if (neighbor.GetIdx() not in visited and 
                neighbor.GetSymbol() == 'C' and 
                neighbor.GetIdx() not in ring_atoms):
                queue.append((neighbor, length + 1))
                max_length = max(max_length, length + 1)
                
    return max_length


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:10036',
                          'name': 'wax ester',
                          'definition': 'A fatty acid ester resulting from the '
                                        'condensation of the carboxy group of '
                                        'a fatty acid with the alcoholic '
                                        'hydroxy group of a fatty alcohol.',
                          'parents': ['CHEBI:35748', 'CHEBI:73702']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.1652892561983471 is too low.\n'
               'True positives: '
               "[('O(C(=O)CCCCCCCCCCCCCCCCCCCCC)CCCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 23'), "
               "('O(CCCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)C(=O)CCCCCCC/C=C\\\\CCCCCCCC', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 19'), "
               "('O(CCCCCCCC/C=C\\\\CCCCCC)C(=O)CCCCCCC/C=C\\\\CCCCCC', 'Wax "
               'ester found with alcohol chain length 16 and acid chain length '
               "17'), "
               "('O(CCCCCCCCCCCCCCCCCC)C(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 19'), ('O(CCCCCCCCCCCC)C(=O)CCCCCCC/C=C\\\\CCCC', 'Wax "
               'ester found with alcohol chain length 12 and acid chain length '
               "15'), ('CCCCCCCCCCCCCCCCOC(=O)CCCCCCC\\\\C=C\\\\CCCCCCCC', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 19'), ('O(CCCCCCCCCCCCCCCCCC)C(=O)CCCCCCCCC/C=C/CCCC', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 17'), "
               "('O(CCCCCCCCCCCC)C(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC', 'Wax "
               'ester found with alcohol chain length 12 and acid chain length '
               "19'), ('O(CCCCCCCC/C=C\\\\CCCCCC)C(=O)CCCCCCC/C=C\\\\CCCC', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 15'), ('O(CCCCCCCC/C=C\\\\CCCC)C(=O)CCCCCCCCCCCCC', "
               "'Wax ester found with alcohol chain length 14 and acid chain "
               "length 15')]\n"
               'False positives: '
               "[('O[C@H]1C[C@H]([C@]2([C@@](C1)(CC[C@]3([C@]4(CC[C@@H]([C@]4(C[C@H]([C@]23[H])O)C)C5COC(C5)=O)O)[H])O)CO)O', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('[H][C@@]12CC[C@]3([H])[C@]([H])(CC[C@@]4(C)[C@@]3([H])CC[C@]4(C)O)[C@@]1(C)COC(=O)C2', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 11'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)OCCCc1cc(OC)c2oc(cc2c1)-c1ccc2OCOc2c1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 19'), ('O1CCCCCCCCC=CCC1=O', 'Wax ester found with "
               "alcohol chain length 12 and acid chain length 13'), "
               "('O=C(O[C@H](C=C(C)C)C/C(=C/CC1=C(O)C(=C(COC(=O)CCCCCCCCCCCCCCC)C=C1OC)C=O)/C)CCCCCCCCCCCCCCC', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 17'), "
               "('[H][C@@]1(CC[C@]2(O)[C@]3([H])CC[C@]4(O)C[C@H](C[C@@H](O)[C@]4(CO)[C@@]3([H])[C@H](O)C[C@]12C)O[C@@H]1O[C@@H](C)[C@H](O)[C@@H](O)[C@H]1O)C1COC(=O)C1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('O(C(=O)CCCCCCC/C=C/C\\\\C=C\\\\CCCCC)CC=1C(=C(O)C(C/C=C(/CC(=O)C=C(C)C)\\\\C)=C(OC)C1)C=O', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 19'), "
               "('O=C1C2=C([C@@]3([C@@H](O)C[C@@H]([C@]3(C1)C)[C@]4(COC(C4)=O)CO)C)[C@@H](O)C[C@@H]5[C@@]2(CCC(C5(C)C)=O)C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('CCCCCCCCCCCCCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 15'), "
               "('O=C(OC[C@H](C1=C2C3=CC=C(C=O)C[C@@H]([C@@]3(CC[C@@]2(C)CC1)C)O)C)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 19'), "
               "('COc1cc2OC(=C)C(=O)Nc2c(c1)C(=O)O[C@H]1COC(=O)C[C@H](N)c2ccc(O[C@@H]3c4ccc1cc4C1=CC=C[C@]31OC1OC(C)(C)C(C(O)C1O)N(C)C)c(Cl)c2', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('C1=2N3C(C=C4[N+]5=C(C=C6N7C8=C(C9=[N+](C(=C1)[C@H]([C@@H]9CCC(OC/C=C(/CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)\\\\C)=O)C)[Mg-2]735)[C@H](C(C8=C6C)=O)C(=O)OC)C(=C4*)*)=C(C2*)*', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 15'), "
               "('ClC1=C2O[C@@H]3C4=C(C=C([C@@H](OC(=O)C5=C6NC(=O)C(=C)OC6=CC(=C5)OC)COC(C[C@@H](C(=C1)C=C2O)N)=O)C=C4)C=7[C@]3(O[C@@H]8OC([C@@H](N(C)C)[C@@H](C8O)O)(C)C)C=CC7', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('O=C1OCC2C=C(C34OC5CC(C3C(C5)C=CC4C(OC)C(C(OC2C(O)COC=CC6=C(CC1)C(=O)OC6=O)=O)OC)O)C', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('O(C(=O)CCC1C(C=2NC1=C3CC(=O)C=4C3=NC(C4C)=CC=5N=C(C=C6N=C(C2)C(=C6C=C)C)C(C5CC)=CO)C)C\\\\C=C(\\\\CC(CC(CCCC(CCCCC)C)C)C)/C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('COC1=C[C@@H](C)[C@@H]2C[C@H]3OC(=O)C[C@H]4[C@@](C)(C[C@@H](O)[C@@H]([C@@]34C)[C@@]2(C)C1=O)C(=O)[C@H]1COC(=O)C1', "
               "'Wax ester found with alcohol chain length 11 and acid chain "
               "length 13'), "
               "('[H][C@@]1(CCC(=O)OC1)[C@@]1([H])CC[C@]2([H])[C@]3([H])CC[C@@]4([H])CCCC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 15'), "
               "('CCCCCCCCCCCCCCCC(=O)OC\\\\C=C(C)\\\\C=C\\\\C=C(C)\\\\C=C\\\\C1=C(C)CCCC1(C)C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 17'), "
               "('ClC1=C2C=CC3=C1C4=CC=C[C@]4(O[C@@H]5OC([C@@H](N(C)C)[C@H]([C@H]5O)O)(C)C)[C@@H]3OC6=C(Cl)C=C([C@H](CC(OC[C@@H]2OC(=O)C7=C8NC(=O)C(=C)OC8=CC(=C7)OC)=O)N)C=C6O', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('O=C1OC[C@@H](C1)C(=C2[C@H](C=C)CC(C2)(C)C)COC(=O)CCC(=O)OC', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 10'), "
               "('CCC1=C(C)C2=Cc3c(C=C)c(C)c4C=C5[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C6=[N+]5[Mg--]5(n34)n3c(=CC1=[N+]25)c(C)c1C(=O)[C-](C(=O)OC)C6=c31', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('CCC1=C(C=O)C2=Cc3c(C=C)c(C)c4C=C5[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C6=[N+]5[Mg--]5(n34)n3c(=CC1=[N+]25)c(C)c1C(=O)[C@H](C(=O)OC)C6=c31', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('ClC1=C2OC3C#CC=C(C(OC(=O)C4=C5NC(=O)C(C)OC5=CC(=C4)OC)COC(CC(C(=C1)C=C2O)N)=O)C#CC=6C3(O[C@@H]7OC([C@@H](N(C)C)C(C7O)O)(C)C)C=CC6', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('O1C2(C3(C(C4(C(CC3=O)C(O)(C)C)C(O)CC(OC4)=O)CCC2(C(OC5OC(C(O)C(O)C5O)CO)C=6C=COC6)C)C)C1C(O)=O', "
               "'Wax ester found with alcohol chain length 10 and acid chain "
               "length 13'), ('O(CCCCCCCCCC)C(=O)CCCCCCC', 'Wax ester found "
               "with alcohol chain length 10 and acid chain length 9'), "
               "('O1C(CC(=O)C2=C1C=CC(=C2N)C(=O)CC(NC(=O)C)COC(=O)CCCCCCC/C=C/CCCCCCCC)(C)C', "
               "'Wax ester found with alcohol chain length 11 and acid chain "
               "length 19'), "
               "('C12=CC=CC=C1N=C3C(=C2)CN4C(C5=C(C=C34)[C@](CC(OC5)=O)(CC)O)=O', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('O(C(=O)CCCCCCCCCCCCCCCCC)CC=1C(=C(O)C(=C(OC)C1)C/C=C(/CC(=O)C=C(C)C)\\\\C)C=O', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 19'), "
               "('O=C(OCC1=C(C=2O[C@](CCC2C(=C1)OC)(CC(=O)C=C(C)C)C)C=O)CCCCCCCCCCCCCCCCC', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 19'), "
               "('CCCCCCCCCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 11'), "
               "('C1(C)(C)C(\\\\C=C\\\\C(=C\\\\C=C\\\\C(=C\\\\COC(CCCCCC)=O)\\\\C)\\\\C)=C(C)CCC1', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 8'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC(OC/C=C(/CC/C=C(/CC/C=C(/CCC=C(C)C)\\\\C)\\\\C)\\\\C)=O)C)[Mg-2]735)[C-](C(C8C6C)=O)C(=O)OC)[C@@H]([C@H]4C)CC)C(=C2C)C(=O)C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 16'), "
               "('COC(=O)[C@H]1C(=O)c2c(C)c3=CC4=[N+]5C(=Cc6c(C=C)c(C)c7C=C8[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C9=[N+]8[Mg--]5(n67)n3c2=C19)C(C)=C4C=C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 16'), "
               "('C\\\\C=C(/C)C(=O)O[C@@H]1[C@@H]2OC[C@@]3(C)[C@H]2[C@](C)([C@@H](O)C[C@H]3OC(C)=O)[C@H]2CC[C@@]3(C)[C@H](C4COC(=O)C4)C(=O)C=C3[C@]12C', "
               "'Wax ester found with alcohol chain length 11 and acid chain "
               "length 13'), "
               "('[H][C@@]12CC[C@](O)(C(=O)COC(=O)CCCCCCC(=O)N(C)CCS([O-])(=O)=O)[C@@]1(C)C[C@H](O)[C@@]1([H])[C@@]2([H])C[C@H](C)C2=CC(=O)C=C[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 9'), "
               "('O=C1OCC2=CC(O)=CC(=C2)CC(=CC=C[C@H](C(C[C@@H]([C@H](C=C([C@H](C1)O)C)C)O)=O)OC)C', "
               "'Wax ester found with alcohol chain length 19 and acid chain "
               "length 20'), "
               "('[H][C@]12CCC=C(C)[C@]1(C)[C@@H](OC(=O)c1cccnc1)[C@H](OC(C)=O)[C@]1(C)O[C@@]3(COC(=O)C3)C[C@H](OC(=O)c3cccnc3)[C@]21C', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 11'), "
               "('CC(O)C1=C2C=C3C(C)=C(C)C4=[N+]3[Mg--]35N6C(=C4)C(C)=C4C(=O)CC(=C64)C4=[N+]3C([C@@H](C)[C@@H]4CCC(=O)OC\\\\C=C(/C)CC\\\\C=C(/C)CCC=C(C)C)=C(C)C(N25)=C1C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 16'), "
               "('O=C1O[C@H]([C@H]([C@@H]2[C@@]3([C@@]([C@@]45OC([C@@H]([C@]6(C4=CC3)COC(=O)CC6)CC5)(C)C)(C)CC2)C)C)CC=C1C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 16'), "
               "('C(C[C@@H](CCC[C@H](C)CCCC(C)C)C)C/C(=C/COC(=O)CC[C@@]1(C=2C3=C4N5C(=CC=6[C@@H]([C@@H](C)C(N6)=CC=7N([Zn]5)C(=C(C)C7C(C)=O)C=C(N2)[C@H]1C)CC)C(=C4C(=O)[C@@H]3C(OC)=O)C)[H])/C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 16'), "
               "('[H][C@]1(COC(=O)C1)[C@@]1([H])CC[C@]2([H])[C@]3([H])CCC4CCCC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('CCC1=C(C)C2=Cc3c(C=C)c(C)c4C=C5[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C6=[N+]5[Mg--]5(n34)n3c(=CC1=[N+]25)c(C)c1C(=O)[C@H](C(=O)OC)C6=c31', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OCCCc1cc(OC)c2oc(cc2c1)-c1ccc2OCOc2c1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 19'), ('O(CCCCCCCCCCCCC)C(=O)CCC1=CC=C(O)C=C1', 'Wax "
               'ester found with alcohol chain length 13 and acid chain length '
               "8'), ('O1CCCCCCCCCCCCC1=O', 'Wax ester found with alcohol "
               "chain length 13 and acid chain length 14'), "
               "('O(C(=O)CCCCCCCCC)C/C=C(\\\\CCC=C(C)C)/C', 'Wax ester found "
               "with alcohol chain length 8 and acid chain length 11'), "
               "('C1=2N3C(C=C4[N+]5=C(C=C6N7C8=C(C9=[N+](C(=C1)[C@H]([C@@H]9CCC(OC/C=C(/CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)\\\\C)=O)C)[Mg-2]735)[C-](C(C8=C6C)=O)C(=O)OC)C(=C4*)*)=C(C2*)*', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 15'), "
               "('O=C1OCC(C1)/C(=C/2\\\\[C@H](C=C)C[C@](C2)(CO)C)/CO', 'Wax "
               'ester found with alcohol chain length 8 and acid chain length '
               "10'), "
               "('O1C(CC(=O)C2=C1C=CC(=C2N)C(=O)CC(NC(=O)C)COC(=O)CCCCCCCCCCCCC(C)C)(C)C', "
               "'Wax ester found with alcohol chain length 11 and acid chain "
               "length 16'), "
               "('C1(CC(OC/C=C(/CC/C=C(/CCC=C(C)C)\\\\C)\\\\C)=O)=C(C)N(C2=C1C=C(C=C2)OC)C(C3=CC=C(Cl)C=C3)=O', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 8'), "
               "('O=C1C2=C([C@@]3(C(=O)C[C@@H]([C@]3([C@H]1O)C)C4(COC(C4)=O)C)C)[C@@H](O)C[C@@H]5[C@@]2(CC[C@@H](C5(C)C)O)C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OCCCc1ccc2oc(cc2c1)-c1ccc2OCOc2c1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 19'), "
               "('CCc1c(C)c2cc3[nH]c(cc4nc([C@@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)[C@@H]4C)c4[C@@H](C(=O)OC)C(=O)c5c(C)c(cc1n2)[nH]c45)c(C)c3C=C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('O=C(OC[C@H](C1=C2C3=CC=C(C=O)C[C@@H]([C@@]3(CC[C@@]2(C)CC1)C)O)C)CCCCCCCCCCCCCCCCC', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 19'), "
               "('O1C(CCC=2C1=C(C(=CC2OC)COC(=O)CCCCCCCCCCCCCCC)C=O)(CC(=O)C=C(C)C)C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 17'), "
               "('CC(O)c1c(C)c2\\\\C=C3/N=C([C@@H](CCC(=O)OC\\\\C=C(/C)CC\\\\C=C(/C)CCC=C(C)C)[C@@H]3C)C3=c4c(C(=O)C3)c([*])c3=CC5=N\\\\C(=C/c1n2[Mg]n43)\\\\C(C)=C5[*]', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 16'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC\\\\C=C(C)\\\\C=C\\\\C=C(C)\\\\C=C\\\\C1=C(C)CCCC1(C)C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 19'), "
               "('O=C1OC[C@]2([C@@]3(O[C@]45O[C@@]6(OC([C@H]([C@@H]([C@]4(C3)C)C)[C@@H]6O5)=O)C)[C@H]([C@@H](O)C[C@H]2C(O)(C)C)C)CC1', "
               "'Wax ester found with alcohol chain length 10 and acid chain "
               "length 13'), "
               "('[H]C(=O)c1c(C=C)c2\\\\C=C3/N=C(C=c4c(C)c5C(=O)[C@H](C(=O)OC)C6=c5n4[Mg]n2c1\\\\C=C1/N=C6[C@@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)[C@@H]1C)C(CC)=C/3C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('C1CCC(=C(/C=C/C(=C\\\\C=C\\\\C(=C\\\\COC(CCCCCCCCCCCCC)=O)\\\\C)/C)C1(C)C)C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 15'), "
               "('Cl[C@@H]1[C@@H](O)C[C@@H](O)/C(=C/C=2N=C(C)SC2)/COC(=O)C[C@H](O)C(C([C@@H]([C@H]([C@H](CCC1)C)O)C)=O)(C)C', "
               "'Wax ester found with alcohol chain length 17 and acid chain "
               "length 20'), "
               "('Cl/C=C/1\\\\C=CC=2C(O)=C(COC(=O)CCCCCCCCCCCCCCC)C=CC2OC1', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 17'), "
               "('COC(=O)[C-]1C(=O)c2c(C)c3=CC4=[N+]5C(=Cc6c(C=C)c(C)c7C=C8[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C9=[N+]8[Mg--]5(n67)n3c2=C19)C(C)=C4C=C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 16'), "
               "('O=C1O[C@H]([C@H]([C@@H]2[C@@]3([C@@](C=4C([C@@]5([C@H](C(OC(=O)C)(C)C)CC4)COC(=O)CC5)=CC3)(C)CC2)C)C)CC=C1C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 16'), "
               "('[H][C@]1(COC(=O)C1)[C@@]1([H])CC[C@]2(O)[C@]3([H])CC[C@]4([H])C[C@H](CC[C@]4(C)[C@@]3([H])CC[C@]12C)O[C@H]1C[C@H](O)[C@H](O[C@H]2C[C@H](O)[C@H](O[C@H]3C[C@H](O)[C@H](O)[C@@H](C)O3)[C@@H](C)O2)[C@@H](C)O1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), ('O1CCCCCCCCC=CCCCCCC1=O', 'Wax ester found with "
               "alcohol chain length 16 and acid chain length 17'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)OCCCc1ccc2oc(cc2c1)-c1ccc2OCOc2c1', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 19'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](COC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC)NC(C)=O', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 19'), "
               "('O1C[C@@]2([C@]3([C@@]([C@@]4([C@@](CC3)(C([C@@H](O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO)CC4)(C)C)[H])C)(CC[C@@]2([C@H]([C@@H](OC(=O)C)C[C@H]6OC(=O)[C@@H](C6)C)C)[H])[H])C)CC1=O', "
               "'Wax ester found with alcohol chain length 10 and acid chain "
               "length 12'), "
               "('COc1cc2OC(=C)C(=O)Nc2c(c1)C(=O)O[C@H]1COC(=O)C[C@H](N)c2cc(O)c(O[C@@H]3c4ccc1cc4C1=CC=C[C@]31OC1OC(C)(C)C(C(O)C1O)N(C)C)c(Cl)c2', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 8'), "
               "('CCCCCCCCCCCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 13'), ('O(CCCCCCCC)C(=O)CCCCCCCCC', 'Wax ester found "
               "with alcohol chain length 8 and acid chain length 11'), "
               "('O1CCCCCCCCCCCCCCCCCCCCCC1=O', 'Wax ester found with alcohol "
               "chain length 22 and acid chain length 23'), "
               "('O1C(CC(=O)C2=C1C=CC(=C2N)C(=O)CC(NC(=O)C)COC(=O)CCCCCCCC/C=C/C=C/CCCCC)(C)C', "
               "'Wax ester found with alcohol chain length 11 and acid chain "
               "length 19'), "
               "('[H][C@@]1(CCC(=O)OC1)[C@@]1([H])CC[C@]2([H])[C@]3([H])CC[C@]4([H])CCCC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 15'), "
               "('O1C(CCC=2C1=C(C(=CC2OC)COC(=O)CCCCCCC/C=C/C/C=C/CCCCC)C=O)(CC(=O)C=C(C)C)C', "
               "'Wax ester found with alcohol chain length 13 and acid chain "
               "length 19'), "
               "('O=C1OCC2=CC(O)=CC(=C2)CC(=CC=C[C@@H](C(C[C@@H]([C@H](C=C([C@H](C1)O)C)C)O)=O)OC)C', "
               "'Wax ester found with alcohol chain length 19 and acid chain "
               "length 20'), "
               "('[H][C@]12C=C(COC(=O)CCCCCCCCCCCCCCC)[C@@H](O)[C@]3(O)[C@@H](OC(=O)C(C)C(C)C)C(C)=C[C@]3([C@H](C)C[C@]3(OC(=O)CCCCCCCCCCC)[C@@]1([H])C3(C)C)C2=O', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 17'), ('O1C=2CCCCCOC(=O)CCC1=C(C2C)C', 'Wax ester found "
               "with alcohol chain length 12 and acid chain length 13'), "
               "('[C@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)(NC(=O)CCCCCCCCCCCCCCCCCCCCCCCCCCCOC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)CO', "
               "'Wax ester found with alcohol chain length 28 and acid chain "
               "length 19'), "
               "('[H][C@]12CCC=C(C)[C@]1(C)[C@@H](OC(=O)c1cccnc1)[C@H](OC(=O)c1cccnc1)[C@]1(C)O[C@@]3(COC(=O)C3)C[C@H](OC(=O)c3cccnc3)[C@]21C', "
               "'Wax ester found with alcohol chain length 9 and acid chain "
               "length 11'), "
               "('C[C@]12C[C@H](O)[C@H]3[C@@H](CCC4=CC(=O)C=C[C@]34C)[C@@H]1CC[C@]2(O)C(=O)COC(=O)CCCc1ccc(cc1)N(CCCl)CCCl', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 9'), ('CCCCCCCCCCCCCCCC(=O)OCC(O)CCCCCCCCCCCCCC', 'Wax "
               'ester found with alcohol chain length 16 and acid chain length '
               "17'), ('O1CCCCCCCCCCCCCCCCCCCC1=O', 'Wax ester found with "
               "alcohol chain length 20 and acid chain length 21'), "
               "('O=C1C(=C2[C@@](CC2)(C)C3=C1C[C@](COC(=O)CCCCCCCCCCCCCCCCC)(C)C3)C', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 19'), "
               "('[H][C@]1(COC(=O)C1)[C@@]1([H])CC[C@]2([H])[C@]3([H])CC[C@]4([H])CCCC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('O=C(OCC(O)([C@@H]1C[C@H]2[C@H](CC[C@H]2C)[C@@](CC1)(O)C)C)CCCCCCCCCCCCCCC', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 17'), "
               "('[H]C(C)=C1C(C)C2=CC3=C(C(C)=O)C(C)=C4C=C5[C@@H](C)[C@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)C6=[N+]5[Mg--]5(N34)N3C(=CC1=[N+]25)C(C)=C1C(=O)[C@H](C(=O)OC)C6=C31', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), "
               "('O=C1C2=C(O[C@](C1)([C@@H](O)[C@H]3COC(C3)=O)CO)C=CC=C2O', "
               "'Wax ester found with alcohol chain length 10 and acid chain "
               "length 12'), ('O(CCC(CCC=C(C)C)C)C(=O)CC(CCCC(C)C)C', 'Wax "
               'ester found with alcohol chain length 8 and acid chain length '
               "9'), "
               "('C12=CC(=C(C=C1N=C3C(=C2)CN4C(C5=C(C=C43)[C@](CC(OC5)=O)(CC)O)=O)F)F', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), "
               "('[H][C@]1(COC(=O)C1)[C@@]1([H])CC[C@]2([H])[C@]3([H])CC[C@@]4([H])CCCC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Wax ester found with alcohol chain length 12 and acid chain "
               "length 14'), ('O(CCCCCCCC)C(=O)CCCCCC', 'Wax ester found with "
               "alcohol chain length 8 and acid chain length 8'), "
               "('O(CCCCCCC(C)C)C(=O)CCCCCC(C)C', 'Wax ester found with "
               "alcohol chain length 8 and acid chain length 9'), "
               "('O=C1OC[C@@H]2CC=C3[C@@H]([C@]2(CC1)C)CC[C@]4([C@H]3CC[C@@H]4[C@@H]([C@@H](O)[C@@H](O[C@@H]5O[C@@H]([C@@H](O)[C@@H]([C@@H]5O)O)CO)[C@@H](C(C)C)C)C)C', "
               "'Wax ester found with alcohol chain length 14 and acid chain "
               "length 16'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)NC(C)=O', "
               "'Wax ester found with alcohol chain length 18 and acid chain "
               "length 19'), "
               "('CCCCCCCCCCCCCCCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 17'), ('O1CCCCCCCCC1=O', 'Wax ester found with alcohol "
               "chain length 9 and acid chain length 10'), "
               "('COC(=O)[C@H]1C(=O)c2c(C)c3C=C4C(C=C)=C(C=O)C5=[N+]4[Mg--]46n3c2C1=C1[C@@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)[C@H](C)C(C=c2c(C)c(C=C)c(=C5)n42)=[N+]61', "
               "'Wax ester found with alcohol chain length 16 and acid chain "
               "length 16'), "
               "('[H][C@]12O[C@@]1(COC(=O)CCCCCCCCCCCCCCC)[C@@H](O)[C@]1(O)[C@@H](OC(=O)C(C)C(C)C)C(C)=C[C@@]11[C@H](C)C[C@]3(OC(=O)CCCCCCCCCCC)[C@]([H])([C@@]2([H])C1=O)C3(C)C', "
               "'Wax ester found with alcohol chain length 8 and acid chain "
               "length 17')]\n"
               "False negatives: [('*C(O*)=O', 'No ester group found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 10,
    'num_false_positives': 46,
    'num_true_negatives': 183779,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.17857142857142858,
    'recall': 0.9090909090909091,
    'f1': 0.2985074626865672,
    'accuracy': 0.9997443373441546}


"""
Classifies: CHEBI:26395 purine nucleotide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_purine_nucleotide(smiles: str):
    """
    Determines if a molecule is a purine nucleotide.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a purine nucleotide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of phosphate group
    phosphate_pattern = Chem.MolFromSmarts('[P](=[O])([O,OH])[O,OH]')
    if not mol.HasSubstructMatch(phosphate_pattern):
        return False, "No phosphate group found"

    # Check for ribose/deoxyribose sugar
    sugar_pattern = Chem.MolFromSmarts('[CH2]O[CH]1[CH][CH][CH]([CH]1)N')
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "No ribose/deoxyribose sugar found"

    # Check for purine base (adenine or guanine pattern)
    adenine_pattern = Chem.MolFromSmarts('c1nc(N)nc2c1ncn2')
    guanine_pattern = Chem.MolFromSmarts('c1nc(N)c2c(n1)N=CN2')
    
    has_adenine = mol.HasSubstructMatch(adenine_pattern)
    has_guanine = mol.HasSubstructMatch(guanine_pattern)
    
    if not (has_adenine or has_guanine):
        return False, "No purine base (adenine/guanine) found"

    if has_adenine and has_guanine:
        return True, "Contains both adenine and guanine bases"
    elif has_adenine:
        return True, "Contains adenine base"
    else:
        return True, "Contains guanine base"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26395',
                          'name': 'purine nucleotide',
                          'definition': 'Any nucleotide that has a purine '
                                        'nucleobase.',
                          'parents': ['CHEBI:26401', 'CHEBI:36976']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183692,
    'num_false_negatives': 24,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998693635829214}


"""
Classifies: CHEBI:190711 epoxy fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_epoxy_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is an epoxy fatty acid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an epoxy fatty acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylate anion group
    carboxylate_pattern = Chem.MolFromSmarts('C(=O)[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion group found"

    # Check for epoxide ring
    epoxide_pattern = Chem.MolFromSmarts('C1OC1')
    if not mol.HasSubstructMatch(epoxide_pattern):
        return False, "No epoxide ring found"

    # Count carbons in main chain
    carbon_count = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C'])
    if carbon_count < 12:  # Minimum length for fatty acids
        return False, "Carbon chain too short for fatty acid"

    # Check for aliphatic nature (allowing unsaturation)
    aromatic_atoms = [atom for atom in mol.GetAtoms() if atom.GetIsAromatic()]
    if aromatic_atoms:
        return False, "Contains aromatic rings"

    # Additional checks for typical fatty acid features
    # Count unsaturations (double bonds)
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bond_count = len(mol.GetSubstructMatches(double_bond_pattern))
    
    # Count additional oxygen-containing groups (like hydroxy, hydroperoxy)
    oxygen_pattern = Chem.MolFromSmarts('[O]')
    oxygen_count = len(mol.GetSubstructMatches(oxygen_pattern))
    
    return True, f"Epoxy fatty acid anion with {double_bond_count} double bonds and {oxygen_count} oxygen-containing groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:190711',
                          'name': 'epoxy fatty acid anion',
                          'definition': 'An epoxy monocarboxylic acid anion '
                                        'resulting from the deprotonation of '
                                        'the carboxy group of any epoxy fatty '
                                        'acid.',
                          'parents': ['CHEBI:190712']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 80,
    'num_true_negatives': 183818,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.03614457831325301,
    'recall': 1.0,
    'f1': 0.06976744186046512,
    'accuracy': 0.9995649833334239}